// TFT display constants

#define TFT_WIDTH   320
#define TFT_HEIGHT    240
#define GRID_WIDTH    300
#define GRID_HEIGHT   200

#define ADC_MAX_VAL   256 //4096
#define ADC_GRID      256 //4096  

#include "stm32f30x.h"
#include "video.h"
#include "gdi.h"
#include "string.h"
#include <stdlib.h>
#include "main.h"

// global capture variables

unsigned char chCapture[NUM_SAMPLES*2] = {0};
// rendered waveform data is stored here for erasing
unsigned char ch1Old[GRID_WIDTH+1] = {0};
unsigned char ch2Old[GRID_WIDTH+1] = {0};
unsigned char sMess[5];
extern volatile u8 Key_mode,Key_plus, Key_minus;
u8 Key_mode_prev=0,Key_plus_prev=0, Key_minus_prev=0;
extern u8 Key_plus_cnt, Key_minus_cnt;

uint16_t sIndex = 0;
int16_t trigLevel = 130;
bool triggerRising = 1;

//volatile bool keepSampling = true;
long samplingTime;
volatile bool hold = false;
// waveform calculated statistics
/*struct Stats {
  bool pulseValid;
  double avgPW;
  float duty;
  float freq;
  float cycle;
  
  bool mvPos;
  float Vrmsf;
  float Vavrf;
  float Vmaxf;
  float Vminf;
} wStats;
*/
double freq1, freq2;
float Vavrf1, Vavrf2;

const char* cplNames[] = {" DC", " AC", "OFF"}; 
uint8_t couplingPos1=0;
uint8_t couplingPos2=0;

const char* rngNames[] = {"   4V", "   2V", "   1V", "400mV", "200mV", "100mV"}; 
uint8_t rangePos1=0;
uint8_t rangePos2=0;

// this represents the drawing offset voltage of channels 1 and 2
volatile int16_t zeroOffsetVoltageA1=120, zeroOffsetVoltageA2=20;

// timebase enumerations and store
const char* tbNames[]={"  5mS", "2.5mS", "  1mS", "500uS", "250uS", "100uS", " 50uS", " 25uS", " 10uS", "  5uS", "2.5uS", "  1uS", "0.5uS", "250nS"}; 
u16 tbCnt[]={             799,     639,     639,     639,     696,     557,     557,     696,    600, 300/*709*/,300/*355*/,  142,     71,     36};
uint8_t currentTimeBase=7;

// out test frequency
const char* tfNames[] = {"T-OFF", "10000", " 5000", " 2500", " 1000", "500Hz", "100Hz"}; 
uint8_t Mode=1, Mode_prev=0;
extern void SetTimerOut(u8 mode);
uint8_t Fout_mode=1;

// grid variables
uint8_t hOffset = (TFT_WIDTH - GRID_WIDTH)/2;
uint8_t vOffset = (TFT_HEIGHT - GRID_HEIGHT)/2;

// prints waveform statistics on screen
bool printStats = true;
// repaint the labels on screen in draw loop
bool paintLabels = true;//false;

const char* trgNames[] = {"TRIG1 ", "TRIG2 ", "EXTT1 ", "EXTT2 ", "SINGL "}; 
uint8_t triggerType;

void drawStats(void);
void clearNDrawSignals(void);
void drawGrid(void);
void drawLabels(void);
void gdiLineV(uint16_t x1, uint16_t y1, uint16_t y2);
void calculateStats(u8 ch);


// ------------------------
void NumToStr(u32 Num)  {
sMess[0]=(Num%100000)/10000+0x30;
sMess[1]=(Num%10000)/1000+0x30;
sMess[2]=(Num%1000)/100+0x30;
sMess[3]=(Num%100)/10+0x30;
sMess[4]=(Num%10)+0x30;
}

// ------------------------
void initDisplay(void)  {
// ------------------------
  vidClearScreen();
  gdiRectangle(0,0,(VID_PIXELS_X - 1),VID_VSIZE - 1,0);

  // and paint o-scope
    drawGrid();
	// RUN first Rectangle
    gdiRectangle(hOffset, vOffset, GRID_WIDTH+hOffset, GRID_HEIGHT+vOffset,0);
    gdiRectangle(8,3,38,13,GDI_ROP_COPY);
}

// ------------------------
void drawWaves(void)  {
// ------------------------
  // clear and draw signal traces
  if (!hold) clearNDrawSignals();
  
  // if requested update the stats
//  if(printStats)
    drawStats();

  // if label repaint requested - do so now
  if(paintLabels) {
    drawLabels();
    paintLabels = false;
  }
}

// ------------------------
void clearNDrawSignals(void)  {
// ------------------------
static bool wavesOld[2] = {false,false};
// this represents the offset voltage at ADC input (1.75V), when Analog input is zero
int16_t zeroVoltageA1S = 128;
int16_t zeroVoltageA2S = 128;
bool wavesSnap[2];
int16_t val1, val2;
unsigned int i,j;
unsigned int k;
int32_t valt, kt, ktd;

  if (couplingPos1==2) wavesSnap[0]=0; else wavesSnap[0]=1;
  if (couplingPos2==2) wavesSnap[1]=0; else wavesSnap[1]=1;

  for( i = 1; i < NUM_SAMPLES; i+=2) {valt=(chCapture[i]-ADC_GRID/2)*50/43;chCapture[i]=valt+ADC_GRID/2;}
		
  if (rangePos1==4)  for( i = 0; i < NUM_SAMPLES; i+=2) {valt=chCapture[i]*2-zeroVoltageA1S;   if (valt>255) valt=255; if (valt<0) valt=0; chCapture[i]=valt;}
  if (rangePos1==5)  for( i = 0; i < NUM_SAMPLES; i+=2) {valt=chCapture[i]*4-zeroVoltageA1S*3; if (valt>255) valt=255; if (valt<0) valt=0; chCapture[i]=valt;}
  if (rangePos2==4)  for( i = 1; i < NUM_SAMPLES; i+=2) {valt=chCapture[i]*2-zeroVoltageA2S;   if (valt>255) valt=255; if (valt<0) valt=0; chCapture[i]=valt;}
  if (rangePos2==5)  for( i = 1; i < NUM_SAMPLES; i+=2) {valt=chCapture[i]*4-zeroVoltageA2S*3; if (valt>255) valt=255; if (valt<0) valt=0; chCapture[i]=valt;}
     
j=0;
	if ((triggerType/2)==0) {
    i=0;
    do{
     valt = (100+trigLevel-zeroOffsetVoltageA1)*ADC_GRID/GRID_HEIGHT;
     if(triggerType%2) 
      {if (((chCapture[i+2]<=valt)&&(chCapture[i]>valt))||((chCapture[i+2]<valt)&&(chCapture[i]>=valt))) j=i;}
       else
      {if (((chCapture[i]<=valt)&&(chCapture[i+2]>valt))||((chCapture[i]<valt)&&(chCapture[i+2]>=valt))) j=i;}
     i+=2;
     if (i == (NUM_SAMPLES/2-2*GRID_WIDTH)) j=2;
     } while( j==0);
    } else {
    i=1;
    do{
     valt = (100+trigLevel-zeroOffsetVoltageA2)*ADC_GRID/GRID_HEIGHT;
     if(triggerType%2) 
      {if (((chCapture[i+2]<=valt)&&(chCapture[i]>valt))||((chCapture[i+2]<valt)&&(chCapture[i]>=valt))) j=i;}
       else
      {if (((chCapture[i]<=valt)&&(chCapture[i+2]>valt))||((chCapture[i]<valt)&&(chCapture[i+2]>=valt))) j=i;}
     i+=2;
     if (i == (NUM_SAMPLES/2-2*GRID_WIDTH + 1)) j=3;
     } while( j==0);
	  }		
	  if ((triggerType/2)!=0) j--;
    sIndex=j;

	// go through all the data points
  for( i = 0; i < GRID_WIDTH; i++)  {
    // erase old line segment ch2
    if(wavesOld[1]) {
      val1 = ch2Old[i];
      val2 = ch2Old[i+1];
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1,val1, hOffset + i + 1, val2, GDI_ROP_XOR);
    }
    // erase old line segment ch1
    if(wavesOld[0]) {
      val1 = ch1Old[i];
      val2 = ch1Old[i+1];
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1,val1, hOffset + i + 1, val2, GDI_ROP_COPY);
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1,val1, hOffset + i + 1, val2, GDI_ROP_XOR);
    }
    // draw new segments ch1
    if(wavesSnap[0])  {
      k=2*((((unsigned int)tbCnt[currentTimeBase]*i)/30+5)/10)+j;
//			if (1)
			if (tbCnt[currentTimeBase]>299)
        valt=chCapture[k];
			 else
			 {
				 ktd = tbCnt[currentTimeBase];
				 k = (i*ktd)/300;
				 kt = i*10-(3000*k)/ktd;
				 k*=2;
				 k += j;
				 valt = ((chCapture[k+2] - chCapture[k])*kt)*ktd/3000 + chCapture[k];
			 }
      val1 = ((valt - zeroVoltageA1S) * GRID_HEIGHT)/ADC_GRID;
      val1 =  220-val1-zeroOffsetVoltageA1;
      if (val1<(vOffset+1)) val1=vOffset+1;
      if (val1>(GRID_HEIGHT+vOffset-1)) val1=(GRID_HEIGHT+vOffset-1);
      k=2*((((int32_t)tbCnt[currentTimeBase]*(i+1))/30+5)/10)+j;
//      if (1)
			 if (tbCnt[currentTimeBase]>299)
        valt=chCapture[k];
			 else
			 {
				 ktd = tbCnt[currentTimeBase];
				 k = (i+1)*ktd/300;
				 kt = (i+1)*10-(3000*k)/ktd;
				 k*=2;
				 k += j;
				 valt = ((chCapture[k+2] - chCapture[k])*kt)*ktd/3000 + chCapture[k];
			 }
      val2 = ((valt - zeroVoltageA1S) * GRID_HEIGHT)/ADC_GRID;
      val2 = 220-val2-zeroOffsetVoltageA1;
      if (val2<(vOffset+1)) val2=vOffset+1;
      if (val2>(GRID_HEIGHT+vOffset-1)) val2=(GRID_HEIGHT+vOffset-1);
      ch1Old[i] = val1;
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1, val1, hOffset + i + 1, val2, GDI_ROP_COPY);
       else ch1Old[i+1] = val2;
    }
    // draw new segments ch2
    if(wavesSnap[1])  {
      k=2*((((unsigned int)tbCnt[currentTimeBase]*i)/30+5)/10)+j+1;
//			if (1)
			if (tbCnt[currentTimeBase]>299)
        valt=chCapture[k];
			 else
			 {
				 ktd = tbCnt[currentTimeBase];
				 k = (i*ktd)/300;
				 kt = i*10-(3000*k)/ktd;
				 k*=2;
				 k += j+1;
				 valt = ((chCapture[k+2] - chCapture[k])*kt)*ktd/3000 + chCapture[k];
			 }
      val1 = ((valt - zeroVoltageA2S) * GRID_HEIGHT)/ADC_GRID;
      val1 =  220-val1-zeroOffsetVoltageA2;
      if (val1<(vOffset+1)) val1=vOffset+1;
      if (val1>(GRID_HEIGHT+vOffset-1)) val1=(GRID_HEIGHT+vOffset-1);
      k=2*((((int32_t)tbCnt[currentTimeBase]*(i+1))/30+5)/10)+j+1;
//      if (1)
			 if (tbCnt[currentTimeBase]>299)
        valt=chCapture[k];
			 else
			 {
				 ktd = tbCnt[currentTimeBase];
				 k = (i+1)*ktd/300;
				 kt = (i+1)*10-(3000*k)/ktd;
				 k*=2;
				 k += j+1;
				 valt = ((chCapture[k+2] - chCapture[k])*kt)*ktd/3000 + chCapture[k];
			 }
      val2 = ((valt - zeroVoltageA2S) * GRID_HEIGHT)/ADC_GRID;
      val2 = 220-val2-zeroOffsetVoltageA2;
      if (val2<(vOffset+1)) val2=vOffset+1;
      if (val2>(GRID_HEIGHT+vOffset-1)) val2=(GRID_HEIGHT+vOffset-1);
      ch2Old[i] = val1;
      if (i!=(GRID_WIDTH - 1)) gdiLine(NULL,hOffset + i + 1, val1, hOffset + i + 1, val2, GDI_ROP_COPY);
       else ch2Old[i+1] = val2;
    }
  }

  // store the drawn parameters to old storage
  wavesOld[0] = wavesSnap[0];
  wavesOld[1] = wavesSnap[1];
	drawGrid();
}

void gdiLineV(uint16_t x1, uint16_t y1, uint16_t y2) {
 gdiLine(NULL, x1, y1, x1 ,y1+y2, GDI_ROP_COPY);
}

void gdiLineH(uint16_t x1, uint16_t y1, uint16_t x2) {
 gdiLine(NULL, x1, y1, x2+x1 ,y1, GDI_ROP_COPY);
}

// ------------------------
void drawGrid(void) {
  uint8_t hPacing = GRID_WIDTH / 15;
  uint8_t vPacing = GRID_HEIGHT / 10;
  int i,j;
  
  for( i = 1; i < 15; i++)
  for( j = vOffset; j <(GRID_HEIGHT + vOffset); j+=5)
      gdiPoint(NULL, i * hPacing + hOffset, j, GDI_ROP_COPY);

  for( i = 1; i < 10; i++)
  for( j = hOffset; j <(GRID_WIDTH + hOffset); j+=5)
      gdiPoint(NULL, j , i * vPacing + vOffset, GDI_ROP_COPY);
}

void DrawMode(u8 mode)
{
    switch(mode)
  {
    case 0:
    gdiRectangle(8,3,38,13,GDI_ROP_XOR); //RUN
    break;
    case 1:
    gdiRectangle(50,3,90,13,GDI_ROP_XOR); //TRIGx
    break;
    case 2:
    gdiRectangle(98,3,134,13,GDI_ROP_XOR); //25mS
    break;
    case 3:
    gdiRectangle(150,3,184,13,GDI_ROP_XOR); //T-OFF
    break;
    case 4:
    gdiRectangle(10,221,159,238,GDI_ROP_XOR); //<0> 1
    break;
    case 5:
    gdiRectangle(10,223,30,233,GDI_ROP_XOR); //DC
    break;
    case 6:
    gdiRectangle(32,223,64,233,GDI_ROP_XOR); //Range1
    break;
    case 7:
    gdiRectangle(162,221,310,238,GDI_ROP_XOR); //<0> 2
    break;
    case 8:
    gdiRectangle(162,223,182,233,GDI_ROP_XOR); //DC
    break;
    case 9:
    gdiRectangle(184,223,216,233,GDI_ROP_XOR); //Range2
    break;
    case 10:
    gdiRectangle(1,3,9,237,GDI_ROP_XOR);  //TRIG_L
    break;
  }
}

// ------------------------
void drawLabels() {
// ------------------------
int cPos, trigX;

  // paint run/hold information
  trigX = 12;
  if(hold)
   gdiDrawTextEx(trigX, 4, "HOLD", GDI_ROP_COPY);
  else
   gdiDrawTextEx(trigX, 4, " RUN", GDI_ROP_COPY);

  // draw trigger level on left side
   gdiDrawTextEx(3,210 - trigLevel, " ", GDI_ROP_COPY);
   gdiDrawTextEx(3,222 - trigLevel, " ", GDI_ROP_COPY);
   gdiDrawTextEx(3,216 - trigLevel, "|", GDI_ROP_COPY);

  // draw zero level on right side
   gdiDrawTextEx(312,210 - zeroOffsetVoltageA1, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,222 - zeroOffsetVoltageA1, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,216 - zeroOffsetVoltageA1, "*", GDI_ROP_COPY);

  // draw zero level on right side
   gdiDrawTextEx(312,210 - zeroOffsetVoltageA2, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,222 - zeroOffsetVoltageA2, " ", GDI_ROP_COPY);
   gdiDrawTextEx(312,216 - zeroOffsetVoltageA2, "*", GDI_ROP_COPY);

  // draw trigger
  trigX += 40;
  gdiDrawTextEx(trigX, 4, (pu8)trgNames[triggerType/2], GDI_ROP_COPY);
  // draw trigger edge
  trigX += 30;
  cPos=12;
  if(triggerType%2) { gdiLineH(trigX + 2, cPos, 2); gdiLineV(trigX + 2, cPos -8, 8); gdiLineH(trigX, cPos - 8, 2); }
   else  { gdiLineH(trigX, cPos, 2); gdiLineV(trigX + 2, cPos - 8, 8); gdiLineH(trigX + 2, cPos - 8, 2); } 
  
  // print new timebase
  trigX += 20;
  gdiDrawTextEx(trigX, 4, (pu8)tbNames[currentTimeBase], GDI_ROP_COPY);

  trigX += 50;
  gdiDrawTextEx(trigX, 4, (pu8)tfNames[Fout_mode], GDI_ROP_COPY);

  // -----------------
  trigX = hOffset + 2;
  cPos=GRID_HEIGHT + vOffset + 4;
  gdiDrawTextEx(trigX, cPos, (pu8)cplNames[couplingPos1], GDI_ROP_COPY);
  trigX += 22;
  gdiDrawTextEx(trigX, cPos, (pu8)rngNames[rangePos1], GDI_ROP_COPY);

  gdiLineV(160, 221, 19);	 
// -----------------
  trigX = hOffset + 154;
  gdiDrawTextEx(trigX, cPos, (pu8)cplNames[couplingPos2], GDI_ROP_COPY);
  trigX += 22;
  gdiDrawTextEx(trigX, cPos, (pu8)rngNames[rangePos2], GDI_ROP_COPY);

  if (Mode!=Mode_prev)
  {
    DrawMode(Mode_prev);
    Mode_prev=Mode;
    DrawMode(Mode);
  }
}

// ------------------------
void drawStats(void)  {
// ------------------------
//  static long lastCalcTime = 0;
//  bool clearStats = false;

  calculateStats(0);
  calculateStats(1);

   NumToStr(freq1/100);
	 if (sMess[0]==0x30) sMess[0]=' ';
   gdiDrawTextEx(104, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   NumToStr(freq1);
   gdiDrawTextEx(116, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   gdiDrawTextEx(146, 224, "Hz", GDI_ROP_COPY);

   NumToStr(freq2/100);
	 if (sMess[0]==0x30) sMess[0]=' ';
   gdiDrawTextEx(256, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   NumToStr(freq2);
   gdiDrawTextEx(268, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   gdiDrawTextEx(298, 224, "Hz", GDI_ROP_COPY);
	
   if (Vavrf1<0) {Vavrf1*=-1; NumToStr(Vavrf1); sMess[0]='-';} else {NumToStr(Vavrf1); sMess[0]=' ';}
	 if (sMess[1]==0x30) {sMess[1]=sMess[2]; sMess[2]='.'; } else { sMess[4]=sMess[3]; sMess[3]='.';}
	 gdiDrawTextEx(66, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   gdiDrawTextEx(96, 224, "V", GDI_ROP_COPY);

   if (Vavrf2<0) {Vavrf2*=-1; NumToStr(Vavrf2); sMess[0]='-';} else {NumToStr(Vavrf2); sMess[0]=' ';}
	 if (sMess[1]==0x30) {sMess[1]=sMess[2]; sMess[2]='.'; } else { sMess[4]=sMess[3]; sMess[3]='.';}
	 gdiDrawTextEx(218, 224, (unsigned char *)sMess, GDI_ROP_COPY);
   gdiDrawTextEx(248, 224, "V", GDI_ROP_COPY);
/*  
  // calculate stats once a while
  if(millis() - lastCalcTime > 300) {
    lastCalcTime = millis();  
    calculateStats();
    clearStats = true;
  }

  // draw stat labels
  tft.setTextColor(ILI9341_RED, ILI9341_BLACK);

  tft.setCursor(25, 20);
  tft.print("Freq:");
  tft.setCursor(25, 30);
  tft.print("Cycle:");
  tft.setCursor(25, 40);
  tft.print("PW:");
  tft.setCursor(25, 50);
  tft.print("Duty:");
#ifdef DRAW_TIMEBASE
  tft.setCursor(25, 60);
  tft.print("T/div:");
#endif

  tft.setCursor(240, 20);
  tft.print("Vmax:");
  tft.setCursor(240, 30);
  tft.print("Vmin:");
  tft.setCursor(240, 40);
  tft.print("Vavr:");
  tft.setCursor(240, 50);
  tft.print("Vpp:");
  tft.setCursor(240, 60);
  tft.print("Vrms:");
  
  // print new stats
  tft.setTextColor(ILI9341_WHITE, ILI9341_BLACK);

  if(clearStats)
    tft.fillRect(60, 20, 50, 50, ILI9341_BLACK);
  
  if(wStats.pulseValid) {
    tft.setCursor(60, 20);
    tft.print((int) wStats.freq);
    tft.setCursor(60, 30);
    tft.print(wStats.cycle); tft.print(" ms");
    tft.setCursor(60, 40);
    tft.print(wStats.avgPW/1000); tft.print(" ms");
    tft.setCursor(60, 50);
    tft.print(wStats.duty); tft.print(" %");
  }
  
  if(clearStats)
    tft.fillRect(270, 20, GRID_WIDTH + hOffset - 270 - 1, 50, ILI9341_BLACK);
  
  drawVoltage(wStats.Vmaxf, 20, wStats.mvPos);
  drawVoltage(wStats.Vminf, 30, wStats.mvPos);
  drawVoltage(wStats.Vavrf, 40, wStats.mvPos);
  drawVoltage(wStats.Vmaxf - wStats.Vminf, 50, wStats.mvPos);
  drawVoltage(wStats.Vrmsf, 60, wStats.mvPos);
*/
}

// ------------------------
void calculateStats(u8 ch) {
// ------------------------
  // extract waveform stats
  int16_t Vmax = -ADC_MAX_VAL, Vmin = ADC_MAX_VAL;
  int64_t sumSamples = 0;
  int64_t sumSquares = 0;
  int64_t freqSumSamples = 0;
  uint16_t k;
  uint16_t fVavr;
  bool dnWave;
  bool firstOne = true;
  uint16_t cHigh = 0;
  uint16_t sCtr;
  int16_t val;
	
  uint16_t sumCW = 0;
  uint16_t sumPW = 0;
  uint16_t numCycles = 0;
  uint16_t numHCycles = 0;
  double tPerSample;
  double avgCycleWidth;
	
  for(k = 0; k < NUM_SAMPLES/2; k++) {
    val = chCapture[k*2+ch]-ADC_GRID/2;
    if(Vmax < val)
      Vmax = val;
    if(Vmin > val)
      Vmin = val;

    sumSamples += val;
    freqSumSamples += chCapture[k*2+ch];
    sumSquares += (val * val);
  }
if (ch==0)
{
 if (couplingPos1==0)
   Vavrf1 = 2*sumSamples/NUM_SAMPLES;
  else
   Vavrf1 = Vmax - Vmin;
 switch(rangePos1)
 {
    case 0: //   4v
			Vavrf1*=(float)4000/255;
			break;
    case 1: //   2v
			Vavrf1*=(float)2000/255;
      break;
    case 2: //   1v
			Vavrf1*=(float)1000/255;
      break;
    case 3: //   0.4v
			Vavrf1*=(float)400/255;
      break;
    case 4: //   0.2v
			Vavrf1*=(float)200/255;
      break;
    case 5: //   0.1v
			Vavrf1*=(float)100/255;
      break;
 }
} else {
 if (couplingPos2==0)
   Vavrf2 = 2*sumSamples/NUM_SAMPLES;
  else
   Vavrf2 = Vmax - Vmin;
 switch(rangePos2)
 {
    case 0: //   4v
			Vavrf2*=(float)4000/255;
			break;
    case 1: //   2v
			Vavrf2*=(float)2000/255;
      break;
    case 2: //   1v
			Vavrf2*=(float)1000/255;
      break;
    case 3: //   0.4v
			Vavrf2*=(float)400/255;
      break;
    case 4: //   0.2v
			Vavrf2*=(float)200/255;
      break;
    case 5: //   0.1v
			Vavrf2*=(float)100/255;
      break;
 }
}

  // find out frequency
  fVavr = 2*freqSumSamples/NUM_SAMPLES;
  dnWave  = (chCapture[sIndex+ch] < fVavr - 3);

  // trigger stopped at sIndex
  for(sCtr = 0, k = sIndex; sCtr < NUM_SAMPLES/2; sCtr++, k+=2) {
    if(k == NUM_SAMPLES)
      k = 0;
    // mark the points where wave transitions the average value
    if(dnWave && (chCapture[k+ch] > fVavr + 3))  {
      if(!firstOne) {
        sumCW += (sCtr - cHigh);
        numCycles++;
      }
      else
        firstOne = false;
      dnWave = false;
      cHigh = sCtr;
    }

    if(!dnWave && (chCapture[k+ch] < fVavr - 3)) {
      if(!firstOne) {
        sumPW += (sCtr - cHigh);
        numHCycles++;
      }
      dnWave = true;
    }
  }
  
  tPerSample = 0;
  switch(currentTimeBase)
  {
    case 13:
    case 12:
    case 11:
      tPerSample = (double)11/104;
      break;
    case 10:
      tPerSample = (double)13/104;
      break;
    case 9:
    case 8:
      tPerSample = (double)13/52;
      break;
    case 7:
      tPerSample = (double)375/696;
      break;
    case 6:
      tPerSample = (double)750/557;
      break;
    case 5:
      tPerSample = (double)1500/557;
      break;
    case 4:
      tPerSample = (double)3750/696;
      break;
    case 3:
      tPerSample = (double)7500/639;
      break;
    case 2:
      tPerSample = (double)15000/639;
      break;
    case 1:
      tPerSample = (double)37500/639;
      break;
    case 0:
      tPerSample = (double)75000/799;
      break;
  }
  avgCycleWidth = (double)sumCW * tPerSample / numCycles;
  
//      NumToStr(tPerSample*100);
//    gdiDrawTextEx(180, 224, (unsigned char *)sMess, GDI_ROP_COPY);
  if (ch==0)      
	  freq1=(double)1000000/avgCycleWidth;
	 else
	  freq2=(double)1000000/avgCycleWidth;
	
  /*
  wStats.avgPW = sumPW * tPerSample / numHCycles;
  wStats.duty = wStats.avgPW * 100 / avgCycleWidth;
  wStats.freq = 1000000/avgCycleWidth;
  wStats.cycle = avgCycleWidth/1000;
  wStats.pulseValid = (avgCycleWidth != 0) && (wStats.avgPW != 0) && ((Vmax - Vmin) > 20);
  
  wStats.mvPos = (rangePos == RNG_50mV) || (rangePos == RNG_20mV) || (rangePos == RNG_10mV);
  wStats.Vrmsf = sqrt(sumSquares/NUM_SAMPLES) * adcMultiplier[rangePos];
  wStats.Vavrf = sumSamples/NUM_SAMPLES * adcMultiplier[rangePos];
  wStats.Vmaxf = Vmax * adcMultiplier[rangePos];
  wStats.Vminf = Vmin * adcMultiplier[rangePos];
  */
}
/*
// ------------------------
void drawVoltage(float volt, int y, boolean mvRange)  {
// ------------------------
  // text is standard 5 px wide
  int numDigits = 1;
  int lVolt = volt;
  
  // is there a negative sign at front
  if(volt < 0)  {
    numDigits++;
    lVolt = -lVolt;
  }
  
  // how many digits before 0
  if(lVolt > 999)
    numDigits++;
  if(lVolt > 99)
    numDigits++;
  if(lVolt > 9)
    numDigits++;
  
  // mv range has mV appended at back
  if(mvRange) {
    numDigits += 1;
    int x = GRID_WIDTH + hOffset - 10 - numDigits * 5;
    tft.setCursor(x, y);
    int iVolt = volt;
//    tft.print(iVolt);
//    tft.print("m");
  }
  else  {
    // non mV range has two decimal pos and V appended at back
    numDigits += 3;
    int x = GRID_WIDTH + hOffset -10 - numDigits * 5;
//    tft.setCursor(x, y);
//    tft.print(volt);
  }

}
*/
void Delay(volatile uint32_t nCount) {
  for (; nCount != 0; nCount--);
}

void VdivCh1(u8 mode)
{
  switch(mode)
  {
    case 0: //   4v
      GPIO_SetBits(GPIOC, GPIO_Pin_14); // =0    1
      GPIO_SetBits(GPIOC, GPIO_Pin_15); // =1      1
      break;
    case 1: //   2v
      GPIO_ResetBits(GPIOC, GPIO_Pin_14); // =1         0
      GPIO_ResetBits(GPIOC, GPIO_Pin_15); // =0       0
      break;
    case 2: //   1v
      GPIO_SetBits(GPIOC, GPIO_Pin_14); // =0             1
      GPIO_ResetBits(GPIOC, GPIO_Pin_15); // =0             0
      break;
    case 3: //   0.4v
    case 4: //   0.2v
    case 5: //   0.1v
      GPIO_ResetBits(GPIOC, GPIO_Pin_14); // =1           0
      GPIO_SetBits(GPIOC, GPIO_Pin_15); // =1          1
      break;
  }
}
void VdivCh2(u8 mode)
{
  switch(mode)
  {
    case 0: //   4v
      GPIO_SetBits(GPIOB, GPIO_Pin_10); // =0    1
      GPIO_SetBits(GPIOB, GPIO_Pin_11); // =1      1
      break;
    case 1: //   2v
      GPIO_ResetBits(GPIOB, GPIO_Pin_10); // =1         0
      GPIO_ResetBits(GPIOB, GPIO_Pin_11); // =0       0
      break;
    case 2: //   1v
      GPIO_SetBits(GPIOB, GPIO_Pin_10); // =0             1
      GPIO_ResetBits(GPIOB, GPIO_Pin_11); // =0             0
      break;
    case 3: //   0.4v
    case 4: //   0.2v
    case 5: //   0.1v
      GPIO_ResetBits(GPIOB, GPIO_Pin_10); // =1           0
      GPIO_SetBits(GPIOB, GPIO_Pin_11); // =1          1
      break;
  }
}

void controlLoop()  {
// ------------------------
  // start by reading the state of analog system
//    if ((Key_mode==1)&(!Key_mode_prev))
//    {
//      Key_mode_prev++;
//      if (Mode!=10) Mode++; else Mode=0;
//      paintLabels = true;
//    } else if ((!Key_mode)&Key_mode_prev){  Key_mode_prev--; Delay(200); }

    if (Key_plus&(!Key_plus_prev))
    {
      Key_plus_prev++;
      if (Key_mode==1)
      {
         if (Mode!=10) Mode++; else Mode=0;
         paintLabels = true;
			} else
      switch(Mode)
      {
        case 0:
          if (hold) hold = false; else hold = true;
          break;
        case 1:
          if (triggerType!=8) triggerType++;
          break;
        case 2:
          if (currentTimeBase!=13) currentTimeBase++;
          initADC();
          break;
        case 3:
          if (Fout_mode!=6) {Fout_mode++; SetTimerOut(Fout_mode);}
          break;
        case 4:
          if (zeroOffsetVoltageA1<200) {zeroOffsetVoltageA1+=2; Key_plus_cnt=0; Key_plus=0; Key_plus_prev=0;} else zeroOffsetVoltageA1=200;
          break;
        case 5:
          if (couplingPos1!=2) couplingPos1++;
          if (couplingPos1==0) {
						 GPIO_SetBits(GPIOC, GPIO_Pin_13); 
					  } else {
						 GPIO_ResetBits(GPIOC, GPIO_Pin_13);
				    }
          break;
        case 6:
          if (rangePos1!=5) rangePos1++;
          VdivCh1(rangePos1);
          break;
        case 7:
          if (zeroOffsetVoltageA2<200) {zeroOffsetVoltageA2+=2; Key_plus_cnt=0; Key_plus=0; Key_plus_prev=0;} else zeroOffsetVoltageA2=200;
          break;
        case 8:
          if (couplingPos2!=2) couplingPos2++;
          if (couplingPos2==0) {
						 GPIO_SetBits(GPIOB, GPIO_Pin_2); 
					  } else {
						 GPIO_ResetBits(GPIOB, GPIO_Pin_2);
				    }
          break;
        case 9:
          if (rangePos2!=5) rangePos2++;
          VdivCh2(rangePos2);
          break;
        case 10:
          if (trigLevel<200) {trigLevel+=2; Key_plus_cnt=0; Key_plus=0; Key_plus_prev=0;}
          break;
      }
        paintLabels = true;
    } else if ((!Key_plus)&Key_plus_prev){  Key_plus_prev--; Delay(200); }

    if (Key_minus&(!Key_minus_prev))
    {
      Key_minus_prev++;
      if (Key_mode==1)
      {
         if (Mode!=0) Mode--; else Mode=10;
         paintLabels = true;
			} else
      switch(Mode)
      {
        case 0:
          if (hold) hold = false; else hold = true;
          break;
        case 1:
          if (triggerType!=0) triggerType--;
          break;
        case 2:
          if (currentTimeBase!=0) currentTimeBase--;
          initADC();
          break;
        case 3:
          if (Fout_mode!=0) {Fout_mode--; SetTimerOut(Fout_mode);}
          break;
        case 4:
          if (zeroOffsetVoltageA1>0) {zeroOffsetVoltageA1-=2; Key_minus_cnt=0; Key_minus=0; Key_minus_prev=0;} else zeroOffsetVoltageA1=0;
          break;
        case 5:
          if (couplingPos1!=0) couplingPos1--;
          if (couplingPos1==0) {
						GPIO_SetBits(GPIOC, GPIO_Pin_13); 
					 } else {
						GPIO_ResetBits(GPIOC, GPIO_Pin_13);
					 }
          break;
        case 6:
          if (rangePos1!=0) rangePos1--;
          VdivCh1(rangePos1);
          break;
        case 7:
          if (zeroOffsetVoltageA2>0) {zeroOffsetVoltageA2-=2; Key_minus_cnt=0; Key_minus=0; Key_minus_prev=0;} else zeroOffsetVoltageA2=0;
          break;
        case 8:
          if (couplingPos2!=0) couplingPos2--;
          if (couplingPos2==0) {
						GPIO_SetBits(GPIOB, GPIO_Pin_2); 
					 } else {
						GPIO_ResetBits(GPIOB, GPIO_Pin_2);
					 }
          break;
        case 9:
          if (rangePos2!=0) rangePos2--;
          VdivCh2(rangePos2);
          break;
        case 10:
          if (trigLevel>0) {trigLevel-=2; Key_minus_cnt=0; Key_minus=0; Key_minus_prev=0;}  else trigLevel=0;
          break;
      }
      paintLabels = true; 
    } else  if ((!Key_minus)&Key_minus_prev){ Key_minus_prev--; Delay(200); }
}

// ------------------------
