/***************************************************************************
 * STM32 VGA demo
 * Copyright (C) 2012 Artekit Italy
 * http://www.artekit.eu
 * Written by Ruben H. Meleca
 
### main.c
 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "stm32f30x.h"                  // Device header

#include "video.h"
#include "gdi.h"

#include "main.h"

extern uint8_t couplingPos, rangePos;
extern uint8_t Mode, Fout_mode;
extern volatile u16 st_TIM3_IRQHandler;
u16 st_TIM3_IRQHandler_prev;
extern volatile u16 st_TIM4_IRQHandler;

//DataBuffer dataBuffer;
extern volatile u8 ADCDMAfilled;
extern bool paintLabels;
volatile u8    DMABufferReady = 0; // Сигнал конца подготовки буфера
extern volatile bool hold;
extern volatile u16 vline;
extern void controlLoop(void);

void RCC_Configuration(void)
{
  RCC_LSEConfig(RCC_LSE_OFF);
  /* TIM1,2,3,4, ADC1,2, SPI1, DMA, GPIOA, GPIOB, GPIOC and AFIO clocks enable */
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_ADC12 | RCC_AHBPeriph_ADC34 | RCC_AHBPeriph_DMA1 |
                        RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB | RCC_AHBPeriph_GPIOC, ENABLE);
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2 | RCC_APB1Periph_TIM3 | RCC_APB1Periph_TIM4, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1 | RCC_APB2Periph_TIM1, ENABLE);
}

void GPIO_Configuration(void)
{
/* Initialize LED which connected to PC13 */
    GPIO_InitTypeDef  GPIO_InitStructureA;
    GPIO_InitTypeDef  GPIO_InitStructureB;
    GPIO_InitTypeDef  GPIO_InitStructureC;

//#define AN_CH1      PA0   // analog channel 1
//#define AN_CH2      PA1   // analog channel 2
    GPIO_InitStructureA.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_7;
    GPIO_InitStructureA.GPIO_Mode = GPIO_Mode_AN;
    GPIO_InitStructureA.GPIO_PuPd = GPIO_PuPd_NOPULL ;
    GPIO_Init(GPIOA, &GPIO_InitStructureA);
//#define Fout   PA15 - TIM2_CH1:
    GPIO_InitStructureA.GPIO_Pin = GPIO_Pin_15;
    GPIO_InitStructureA.GPIO_Mode = GPIO_Mode_AF;
    GPIO_InitStructureA.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructureA.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_InitStructureA.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_PinAFConfig(GPIOA, GPIO_PinSource15, GPIO_AF_1);
    GPIO_Init(GPIOA, &GPIO_InitStructureA);
// key_shift  key_plus  key_minus
		GPIO_PinAFConfig(GPIOB, GPIO_Pin_3, GPIO_AF_0);
    GPIO_InitStructureB.GPIO_Pin = GPIO_Pin_3|GPIO_Pin_4|GPIO_Pin_7;
    GPIO_InitStructureB.GPIO_Mode = GPIO_Mode_IN;
    GPIO_InitStructureB.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructureB.GPIO_PuPd = GPIO_PuPd_UP;
    GPIO_Init(GPIOB, &GPIO_InitStructureB);
// key_A AC/DC  A1  B1
    GPIO_InitStructureB.GPIO_Pin = GPIO_Pin_2|GPIO_Pin_10|GPIO_Pin_11;
    GPIO_InitStructureB.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructureB.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructureB.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructureB.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOB, &GPIO_InitStructureB);
// key_B AC/DC  A2  B2
    GPIO_InitStructureC.GPIO_Pin = GPIO_Pin_13|GPIO_Pin_14|GPIO_Pin_15;
    GPIO_InitStructureC.GPIO_Mode = GPIO_Mode_OUT;
    GPIO_InitStructureC.GPIO_Speed = GPIO_Speed_2MHz;
    GPIO_InitStructureC.GPIO_OType = GPIO_OType_PP;
    GPIO_InitStructureC.GPIO_PuPd = GPIO_PuPd_NOPULL;
    GPIO_Init(GPIOC, &GPIO_InitStructureC);
}

void SetTimerOut(u8 mode)
{
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
TIM_OCInitTypeDef TIM_OCInitStructure;

  TIM_Cmd(TIM2, DISABLE);
	
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  switch(mode)
	{
		case 0:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(5000*2); //500Khz
		  break;
		case 1:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(100*2); //10Khz
		  break;
		case 2:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(50*2);  //5Khz
		  break;
		case 3:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(25*2);  //2.5Khz
		  break;
		case 4:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(10*2);  //1Khz
		  break;
		case 5:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(5*2);   //500Hz
		  break;
		case 6:
      TIM_TimeBaseStructure.TIM_Period = 1040000/(1*2);  //100Hz
		  break;
	}
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Toggle;//TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 50;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
//  if (mode)
	{
   TIM_OC1Init(TIM2, &TIM_OCInitStructure);
   TIM_CtrlPWMOutputs(TIM2, ENABLE);
   TIM_Cmd(TIM2, ENABLE);
	}
}
int main(void)
{
  RCC_Configuration();
  GPIO_Configuration();
  vidInit();
//  sysInitSystemTimer();

  GPIO_SetBits(GPIOB, GPIO_Pin_2);  // LED_A AC/DC=1
  GPIO_SetBits(GPIOB, GPIO_Pin_10); // =1
  GPIO_SetBits(GPIOB, GPIO_Pin_11); // =1
	
  GPIO_SetBits(GPIOC, GPIO_Pin_13); // LED_B AC/DC=1
  GPIO_SetBits(GPIOC, GPIO_Pin_14); // =1
  GPIO_SetBits(GPIOC, GPIO_Pin_15); // =1

  GPIO_ResetBits(GPIOA, GPIO_Pin_15); // =0 Fout
	
  initADC();

  initDisplay();

  SetTimerOut(1);
	
  while(1)
  {
//		if(st_TIM3_IRQHandler!=st_TIM3_IRQHandler_prev)	
//		{
			while(vline);
			controlLoop(); 
//			st_TIM3_IRQHandler_prev=st_TIM3_IRQHandler;
//		}
		if(st_TIM4_IRQHandler%10==0)
		{
			if (!hold)
			{
		   startADCDMA();
       while (!ADCDMAfilled);
			}
			while(vline);
			drawWaves();
 		}
  }
}
