/***************************************************************************
 * STM32 VGA demo
 * Copyright (C) 2012 Artekit Italy
 * http://www.artekit.eu
 * Written by Ruben H. Meleca
 
### video.c
 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include "stm32f30x.h"
#include "video.h"

#define VTOTAL  42              /* Total bytes to send through SPI */
__align(4) u8 fb[VID_VSIZE][VID_HSIZE+2]; /* Frame buffer */
volatile u16 vline = 0;        /* The current line being drawn */
static volatile u32 vflag = 0;        /* When 1, the SPI DMA request can draw on the screen */
static volatile u32 vdraw = 0;        /* Used to increment vline every 3 drawn lines */ 
volatile u16 st_TIM4_IRQHandler;
volatile u8 Key_mode=0,Key_plus=0, Key_minus=0;
u8 Key_mode_cnt=0,Key_plus_cnt=0, Key_minus_cnt=0;

void TIMER_Configuration(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  NVIC_InitTypeDef nvic;
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_OCInitTypeDef TIM_OCInitStructure;
  u32 TimerPeriod = 0;
  u16 Channel2Pulse = 0, Channel3Pulse = 0, Channel4Pulse = 0;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6; //PORT_B5 - T3C3 remap, PORT_B6 - T4C1 PVV
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource5, GPIO_AF_2);
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_2);
	
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  
  /*
    VGA 640x480 @ 60 Hz
    Vertical refresh  31.469 kHz
    Pixel freq.     24.0 MHz (12*8=96)/4
    Pixel freq.     27.0 MHz (12*9=108)/4

    1 system tick @ 96Mhz = 0,01041(6) us
    1 system tick @ 108Mhz = 0,009(259) us
  */

  /*
    Horizontal timing
    -----------------
    
    Timer 3 period = 31476 Hz
    
    Timer 3 channel 2 generates a pulse for HSYNC each 31.77 us.
    31.77 us = Visible area + Front porch + Sync pulse + Back porch.
    HSYNC is 3.77 us long, so the math to do is:
    3.77us / 0,010416us = 362 system ticks.   //HS
    3.77us / 0,009259us = 407 system ticks.   //HS

    Timer 3 channel 3 generates a pulse equal to HSYNC + back porch.
    This interrupt will fire the DMA request to draw on the screen if vflag == 1.
    Since firing the DMA takes more or less 800ns, we'll add some extra time.
    The math for HSYNC + back porch is:
    (3.77us + 1,89us - dma) / 0,010416us = +-466 system ticks   //HSBP
    (3.77us + 1,89us - dma) / 0,009259us = +-525 system ticks   //HSBP
  
    Horizontal timing info
    ----------------------

                  Dots  us
    --------------------------------------------    
    Visible area  640   25.17
    Front porch   8+8   0.94
    Sync pulse    96    3.77
    Back porch    40+8  1.89
    Whole line    800   31.77  //WL
  
  */

// VGA 640x480 @ 60 Hz    Pixel freq.     24.0 MHz (12*8=96)/4
#define WL  3050+100   // 96000000/31476
#define HS  362
#define HSBP  466+25

// VGA 640x480 @ 60 Hz    Pixel freq.     27.0 MHz (12*9=108)/4
//#define WL  (3430-300)   //  108000000/31476
//#define HS  407
//#define HSBP  (525-30)

  /*
    SVGA 800x600 @ 56 Hz
    Vertical refresh  35.15625 kHz
    Pixel freq.     36.0 MHz
    
    1 system tick @ 72Mhz = 0,0138 us
  */

  /*
    Horizontal timing
    -----------------
    
    Timer 3 period = 35156 Hz       // move to TMR3_C4->C3 PVV  
    
    Timer 3 channel 4 generates a pulse for HSYNC each 28.4 us.
    28.4 us = Visible area + Front porch + Sync pulse + Back porch.
    HSYNC is 2 us long, so the math to do is:
    2us / 0,0138us = 144 system ticks.    //HS
    
    Timer 4 channel 3 generates a pulse equal to HSYNC + back porch.
    This interrupt will fire the DMA request to draw on the screen if vflag == 1.
    Since firing the DMA takes more or less 800ns, we'll add some extra time.
    The math for HSYNC + back porch is:
    (2us + 3,55us - dma) / 0,0138us = +-350 system ticks    //HSBP
  
    Horizontal timing info
    ----------------------

                  Dots  us
    --------------------------------------------    
    Visible area  800   22.222222222222
    Front porch   24    0.66666666666667
    Sync pulse    72    2
    Back porch    128   3.5555555555556
    Whole line    1024  28.444444444444  //WL
  
  */
// SVGA 800x600 @ 56 Hz
//#define WL  2048
//#define HS  144
//#define HSBP  352

  TimerPeriod = WL;
  Channel4Pulse = HS;   /* HSYNC */
  Channel3Pulse = HSBP; /* HSYNC + BACK PORCH */
  
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = Channel4Pulse;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Reset;
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Set;

  TIM_OC2Init(TIM3, &TIM_OCInitStructure);
  
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Inactive;
  TIM_OCInitStructure.TIM_Pulse = Channel3Pulse;
  TIM_OC3Init(TIM3, &TIM_OCInitStructure);

  /* TIM3 counter enable and output enable */
  TIM_CtrlPWMOutputs(TIM3, ENABLE);

  /* Select TIM3 as Master */
  TIM_SelectMasterSlaveMode(TIM3, TIM_MasterSlaveMode_Enable);
  TIM_SelectOutputTrigger(TIM3, TIM_TRGOSource_Update);
  
  /*
    Vertical timing  =====  VGA 640x480 @ 60 Hz
    ---------------
    
    Polarity of vertical sync pulse is positive.

            Lines
    ------------------------------
    Visible area  480
    Front porch   2+8
    Sync pulse    2    // SPV
    Back porch    25+8 // SPBPV
    Whole frame   525  // WFV
    
  */

// VGA 640x480 @ 60 Hz
#define WFV  525
#define SPV  2
#define SPBPV  35

  /*
    Vertical timing  =====  SVGA 800x600 @ 56 Hz
    ---------------
    
    Polarity of vertical sync pulse is positive.

            Lines
    ------------------------------
    Visible area  600
    Front porch   1
    Sync pulse    2    // SPV
    Back porch    22   // SPBPV
    Whole frame   625  // WFV
    
  */

// SVGA 800x600 @ 56 Hz
//#define WFV  625
//#define SPV  2
//#define SPBPV 24

  /* VSYNC (TIM4_CH2) and VSYNC_BACKPORCH (TIM4_CH3) */  // move to TMR4 PVV
  /* Channel 2 and 3 Configuration in PWM mode */
  TIM_SelectSlaveMode(TIM4, TIM_SlaveMode_Gated);
  TIM_SelectInputTrigger(TIM4, TIM_TS_ITR2/*TIM_TS_ITR0*/);
  
  TimerPeriod = WFV;    /* Vertical lines */
  Channel2Pulse = SPV;    /* Sync pulse */
  Channel3Pulse = SPBPV;  /* Sync pulse + Back porch */
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_Period = TimerPeriod;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;

  TIM_TimeBaseInit(TIM4, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = Channel2Pulse;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCNPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Reset;
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Set;
  TIM_OC1Init(TIM4, &TIM_OCInitStructure);
  
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Inactive;
  TIM_OCInitStructure.TIM_Pulse = Channel3Pulse;
  TIM_OC3Init(TIM4, &TIM_OCInitStructure);

  /*  TIM4 counter enable and output enable */
  TIM_CtrlPWMOutputs(TIM4, ENABLE);

  /* Interrupt TIM2 */
  nvic.NVIC_IRQChannel = TIM4_IRQn;
    nvic.NVIC_IRQChannelPreemptionPriority = 1;
    nvic.NVIC_IRQChannelSubPriority = 0;
    nvic.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&nvic);
  TIM_ITConfig(TIM4, TIM_IT_CC3, ENABLE);

  /* Interrupt TIM3 */
  nvic.NVIC_IRQChannel = TIM3_IRQn;
    nvic.NVIC_IRQChannelPreemptionPriority = 1;
    nvic.NVIC_IRQChannelSubPriority = 0;
    nvic.NVIC_IRQChannelCmd = ENABLE;

  NVIC_Init(&nvic);
  TIM_ITConfig(TIM3, TIM_IT_CC3, ENABLE);
  
  TIM_Cmd(TIM4, ENABLE);
  TIM_Cmd(TIM3, ENABLE);
}

void SPI_Configuration(void)
{
  NVIC_InitTypeDef nvic;
  SPI_InitTypeDef SPI_InitStructure;
  DMA_InitTypeDef DMA_InitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;
  
  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
//  GPIO_PinRemapConfig(GPIO_Remap_SPI1, ENABLE);
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_5);
  
  SPI_Cmd(SPI1, DISABLE);
  DMA_DeInit(DMA1_Channel3);

  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&SPI1->DR;
  DMA_InitStructure.DMA_MemoryBaseAddr = (u32) &fb[0][0];
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = VTOTAL;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word; //DMA_MemoryDataSize_Byte;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
//  DMA_InitStructure.DMA_Priority = DMA_Priority_Low;
  DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init(DMA1_Channel3, &DMA_InitStructure);

  SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Tx;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_8;
  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 7;
  SPI_Init(SPI1, &SPI_InitStructure);
  
  SPI_CalculateCRC(SPI1, DISABLE);
  SPI_Cmd(SPI1, ENABLE);
  
  SPI1->CR2 |= SPI_I2S_DMAReq_Tx;
  
  nvic.NVIC_IRQChannel = DMA1_Channel3_IRQn;
  nvic.NVIC_IRQChannelPreemptionPriority = 0;
  nvic.NVIC_IRQChannelSubPriority = 0;
  nvic.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&nvic);
  
  DMA1_Channel3->CCR &= ~1;
  DMA1_Channel3->CNDTR = VTOTAL;
  DMA1_Channel3->CMAR = (u32) &fb[0][0];
  
  DMA_ITConfig(DMA1_Channel3, DMA_IT_TC, ENABLE);
}

//*****************************************************************************
//  This irq is generated at the end of the horizontal back porch.
//  Test if inside a valid vertical start frame (vflag variable), 
//  and start the DMA to output a single frame buffer line through the SPI device.
//*****************************************************************************
//__irq void TIM1_CC_IRQHandler(void)
__irq void TIM3_IRQHandler(void)
{
  if (vflag)
  {
    DMA1_Channel3->CCR = 0x93;
  }
  TIM3->SR &= ~TIM_IT_CC3;
}

//*****************************************************************************
//  This irq is generated at the end of the vertical back porch.
//  Sets the 'vflag' variable to 1 (valid vertical frame).
//*****************************************************************************
__irq void TIM4_IRQHandler(void)
{
  vflag = 1;
  TIM4->SR &= ~TIM_IT_CC3; //0xFFF7; //~TIM_IT_CC3;
}

//*****************************************************************************
//  This interrupt is generated at the end of every line.
//  It will increment the line number and set the corresponding line pointer
//  in the DMA register.
//*****************************************************************************
__irq void DMA1_Channel3_IRQHandler(void)
{ 
  DMA1->IFCR = DMA1_IT_TC3;
  DMA1_Channel3->CCR = 0x92;
  DMA1_Channel3->CNDTR = VTOTAL;
  
  vdraw++;
  
  if (vdraw == 2)
  {
    vdraw = 0;

    vline++;
    
    if (vline == VID_VSIZE)
    {
      vdraw = vline = vflag = 0;
      DMA1_Channel3->CMAR = (u32) &fb[0][0];
      st_TIM4_IRQHandler++;
      if (!GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_7))  Key_mode_cnt++; else {Key_mode_cnt=0; Key_mode=0;}
    	if (Key_mode_cnt>2) {Key_mode_cnt=2; Key_mode=1;}
      if (!GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_4))  Key_plus_cnt++; else {Key_plus_cnt=0; Key_plus=0;}
    	if (Key_plus_cnt>2) {Key_plus_cnt=2; Key_plus=1;}
    	if (!GPIO_ReadInputDataBit(GPIOB,GPIO_Pin_3)) Key_minus_cnt++; else {Key_minus_cnt=0; Key_minus=0;}
    	if (Key_minus_cnt>2) {Key_minus_cnt=2; Key_minus=1;}
    } else {
      DMA1_Channel3->CMAR += VTOTAL;
    }
  }
}

void vidClearScreen(void)
{
  u16 x, y;

  for (y = 0; y < VID_VSIZE; y++)
  {
    for (x = 0; x < VTOTAL; x++)
    {
      fb[y][x] = 0;
    }
  }
}

void vidInit(void)
{
  SPI_Configuration();
  TIMER_Configuration();
  vidClearScreen();
}
